#!/usr/bin/perl -w
#
#	for use with the Dashboard-Widget "Network Stat"
#	(c) by Pascal Pfiffner 2005/2006

# format:
# Interface-Name | key?value | key?value \n

my @short = qw(lo0 en0 en1 fw0 ppp0);
my %name = qw(lo0 loopback en0 Ethernet en1 Airport fw0 FireWire ppp0 Dialup);
my $curr = 'lo0';
my $active = 0;
my(%interfaces, %ipv4, %ipv6, %prefixlen, %netmask, %broadcast, %MAC, %server_identifier, %subnet_mask, %router, %dns, %ssid, %wireless_channel);

open(IFCONFIG, "/sbin/ifconfig|") or die "name?none|ipv4?no info\n";
while(<IFCONFIG>) {
	if(/^[\w\d]+: flags=/) {
		$curr = substr($_, 0, index($_, ":"));
	}
	$interfaces{$curr} .= $_
}


foreach $key (keys %interfaces) {
	my $val = $interfaces{$key};
	
	if(($val =~ /status:\s?active/) || ($key eq 'ppp0')) {		# active interfaces or a dialup connection
		$active++;
		@infos = split(/\s+/, $val);		# split ifconfig on \s
		
			
		$ipv4{$key} = 'unknown';
		$ipv6{$key} = 'unknown';
		$prefixlen{$key} = '##';
		$netmask{$key} = '';
		$broadcast{$key} = '';
		$MAC{$key} = '';
		$server_identifier{$key} = '';
		$subnet_mask{$key} = '';
		$router{$key} = '';
		$dns{$key} = '';
		$ssid{$key} = '';
		$wireless_channel{$key} = '';
		
		##
		# extract infos from ifconfig
		for($i = 0; $i < @infos; $i++) {
			$ipv4{$key} = $infos[$i + 1] if($infos[$i] eq 'inet');			# IPv4
			$ipv6{$key} = $infos[$i + 1] if($infos[$i] eq 'inet6');			# IPv6
			$prefixlen{$key} = $infos[$i + 1] if($infos[$i] eq 'prefixlen');		# Prefix-length
			$netmask{$key} = $infos[$i + 1] if($infos[$i] eq 'netmask');			# Netmask
			$broadcast{$key} = $infos[$i + 1] if($infos[$i] eq 'broadcast');		# Broadcast
			$MAC{$key} = $infos[$i + 1] if($infos[$i] =~  /ether|lladdr/);				# MAC
		}
		
		$ipv6{$key} =~ s/%.+$//;		# remove interface from IPv6
		
		##
		# more information by ipconfig
		my @ipconfig = split(/\n+/, `ipconfig getpacket $key`);
		foreach (@ipconfig) {
			my $foo = '';
			if(/^server_identifier/) {
				s/server_identifier \(ip\): ?//;
				$server_identifier{$key} = $_;
			}
			elsif(/^subnet_mask/) {
				s/subnet_mask \(ip\): ?//;
				$subnet_mask{$key} = $_;
			}
			elsif(/^router/) {
				$foo = substr($_, index($_, "{") + 1, rindex($_, "}") - 1);
				$router{$key} = join(",", split(/[^\d\.]+/, $foo));
			}
			elsif(/^domain_name_server/) {
				$foo = substr($_, index($_, "{") + 1, rindex($_, "}") - 1);
				$dns{$key} = join(",", split(/[^\d\.]+/, $foo));
			}
		}
		
		##
		# More Wireless Info
		if('en1' eq $key) {
			my @wirelessinfo = split(/\n+/, `system_profiler SPAirPortDataType`);
			foreach (@wirelessinfo) {
				chomp;
				if(/Current Wireless Network/) {
					s/^\s+Current Wireless Network: ?//;
					$ssid{$key} = $_;
				}
				elsif(/Wireless Channel/) {
					s/\D+//g;
					$wireless_channel{$key} = $_;
				}
			}
		}
		
		# print it to STDOUT
		print "ifname?$key|name?".($name{$key} ? $name{$key} : 'unknown')."|ipv4?".$ipv4{$key}."|ipv6?".$ipv6{$key}."|prefixlen?".$prefixlen{$key}."|netmask?".$netmask{$key}."|bcast?".$broadcast{$key}."|MAC?".$MAC{$key}."|server_identifier?".$server_identifier{$key}."|subnet_mask?".$subnet_mask{$key}."|router?".$router{$key}."|dns?".$dns{$key}."|ssid?".$ssid{$key}."|wireless_channel?".$wireless_channel{$key}."\n";
	}
}

print STDOUT "name?none|ipv4?no network\n" unless($active > 0);


